package uy.edu.fing.chesstrack.modulovision.imgproc;

import org.opencv.core.CvType;
import org.opencv.core.Mat;
import org.opencv.core.MatOfPoint2f;
import org.opencv.core.Point;
import org.opencv.core.Rect;
import org.opencv.imgproc.Imgproc;

import uy.edu.fing.chesstrack.modulovision.Calibracion;
import android.util.Log;

public class Homografia implements ImgProcInterface {

	private static final String TAG = "CHESSTRACK::Homografia";
	private final  Mat matrizTransformada;
	private final Rect rectPOI;

	public Homografia() throws Exception {
		super();
		MatOfPoint2f vertices = Calibracion.getInstance().getVertices();
		if (vertices != null){
			Mat src = new Mat(4,1,CvType.CV_32FC2);
			Mat dst = new Mat(4,1,CvType.CV_32FC2);
			Point p1,p2,p3,p4;
			p1 = vertices.toList().get(0);
			p2 = vertices.toList().get(6);
			p3 = vertices.toList().get(42);
			p4 = vertices.toList().get(48);
			double d17 = Math.sqrt(Math.pow((p1.x- p2.x),2) + Math.pow((p1.y- p2.y),2));
			double d749 = Math.sqrt(Math.pow((p2.x- p4.x),2) + Math.pow((p2.y- p4.y),2));
			double d143 = Math.sqrt(Math.pow((p1.x- p3.x),2) + Math.pow((p1.y- p3.y),2));
			double d4349 = Math.sqrt(Math.pow((p3.x- p4.x),2) + Math.pow((p3.y- p4.y),2));

			int media = (int) Math.floor( Math.round((d17+d749+d143+d4349)/4));
			int offset = 2*(media/7);
			src.put(0,0 ,(int)p1.x,(int)p1.y, (int)p2.x,(int)p2.y, (int)p3.x,(int)p3.y, (int)p4.x,(int)p4.y);
			dst.put(0,0, offset,offset ,offset+media,offset , offset,offset+media , offset+media,offset+media );
			Log.i(TAG, "offset= " + offset);
			Log.i(TAG, "media = " + media);

			// EL offset es casi el tamaño de una celda
			rectPOI = new Rect(new Point(0,0), new Point(2*offset+media,2*offset+media));
			Calibracion.getInstance().setRectPOI(rectPOI);
			matrizTransformada = Imgproc.getPerspectiveTransform(src,dst);
		}else{
			throw new Exception(TAG + "No se han calculado los vertices");
		}
	}
	/**
	 * Hace homografia y recorta la imagen
	 */
	@Override
	public Mat procesarImagen(Mat inputFrame) {

		Imgproc.warpPerspective(inputFrame,inputFrame, matrizTransformada, inputFrame.size());
		Mat subMat = inputFrame.submat(rectPOI);
		Mat tmp = Mat.zeros(inputFrame.size(), CvType.CV_8UC4);
		Mat matTMP = tmp.submat(rectPOI);
		subMat.copyTo(matTMP);
		return tmp;

	}


}
